import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.concurrent.TimeUnit;
import java.util.Random;

/**
 * A character which moves in random patterns and interacts with 
 * BobRoss. Both EvilRabbits and GoodRabbits are Rabbits.
 */
public class Rabbit extends Actor implements Movable {
    
    private int speed;
    private int frame;
    private int steps;
    private int vSpeed;
    private int accel;
    private long idleStart;
    private boolean left;
    private String standL, standR;
    private String[] walkL, walkR;

    /**
     * Constructs a Rabbit with specified animation images.
     *
     * @param standL the file name of the image of the object facing left.
     * @param standR the file name of the image of the object facing right.
     * @param walkL the file names of the images of the object walking left.
     * @param walkR the file names of the images of the object walking right.
     */
    public Rabbit(String standR, String standL, String[] walkL, String[] walkR) {
        this.standL = standL;
        this.standR = standR;
        this.walkL = walkL;
        this.walkR = walkR;
        speed = 10;
        frame = 0;
        steps = 0;
        vSpeed = 0;
        accel = 10;
    }

    /**
     * Constructs a Rabbit with specified animation images facing a specified direction.
     * 
     * @param left whether or not the object is facing left.
     * @param standL the file name of the image of the object facing left.
     * @param standR the file name of the image of the object facing right.
     * @param walkL the file names of the images of the object walking left.
     * @param walkR the file names of the images of the object walking right.
     */
     public Rabbit(boolean left, String standL, String standR, String[] walkL, String[] walkR) {
        this.left = left;
        this.standL = standL;
        this.standR = standR;
        this.walkL = walkL;
        this.walkR = walkR;
        speed = 10;
        frame = 0;
        steps = 0;
        vSpeed = 0;
        accel = 10;
    }

    /**
     * Make the Rabbit move in a random directions for a random amount of steps.
     * The Rabbit waits for two seconds after a set of steps is completed.
     */
    public void act() {
        fall();
        if (System.currentTimeMillis() - idleStart < 2000) return;
        if (steps==0) {
            steps = (int)(Math.random() * 31 + 7);
            if(left) setImage(standL);
            else setImage(standR);
            left = Math.random() < 0.5;
            idleStart = System.currentTimeMillis();
        } else {
            if(!frontClear()) left = !left;
            if(left) walkL();
            else walkR();
            steps--;
        }
    }

    /**
     * Moves the charactor left and animates walking.
     */
    public void walkL() {
        left = true;
        if(frame != 6){
            setImage(walkL[frame]);
        } else {
            setImage(walkL[6]);
            frame = 0;
            setLocation(getX() - speed, getY());
            return;
        }
        setLocation(getX() - speed, getY());
        frame++;
    }

    /**
     * Moves the character right and animates walking.
     */
    public void walkR() {
        left = false;
        if(frame != 6){
            setImage(walkR[frame]);
        } else {
            setImage(walkR[6]);
            frame = 0;
            setLocation(getX() + speed, getY());
            return;
        }
        setLocation(getX() + speed, getY());
        frame++;
    }

    /**
     * Moves the charcter down until the floor is reached.
     */
     public void fall() {
        if(onGround()){
            vSpeed = 0;
            return;
        }
        vSpeed += accel;
        setLocation(getX(), getY() + vSpeed);
        Actor ground = getOneIntersectingObject(Tiles.class);
        while(ground != null) {
            setLocation(getX(), getY()-1);
            ground = getOneIntersectingObject(Tiles.class);
            if(!onGround()) {
                break;
            }
        }
    }

    /**
     * @return whether or not the chracter is on the ground.
     */
    public boolean onGround() {
        Actor below = getOneObjectAtOffset(0, getImage().getHeight()/2, Tiles.class);
        return below != null;
    }

    /**
     * @return whether or not the front of the character has a Tiles in front of it.
     */
    public boolean frontClear() {
        Actor front;
        if(left) {
            front = getOneObjectAtOffset(-60, getImage().getHeight()/2, Tiles.class);
        } else {
            front = getOneObjectAtOffset(60, getImage().getHeight()/2, Tiles.class);
        }
        if(front != null) {
            return true;
        }
        return false;
    }

    /**
     * @return the horizontal speed of the character.
     */
    public int getSpeed() {
        return speed;
    }

    /**
     *
     * @param speed the horizontal speed of the character.
     */
    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /**
     * @return the animation frame that the charcter is in.
     */
    public int getFrame() {
        return frame;
    }

    /**
     *
     * @param frame the animation frame that the charcter is in.
     */
    public void setFrame(int frame) {
        this.frame = frame;
    }

    /**
     * @return the number of steps remaining in the set of steps.
     */
    public int getSteps() {
        return steps;
    }

    /**
     * @param steps the number of steps remaining in the set of steps.
     */
    public void setSteps(int steps) {
        this.steps = steps;
    }

    /**
     * @return the falling speed of the character.
     */
    public int getvSpeed() {
        return vSpeed;
    }

    /**
     * @param vSpeed the falling speed of the character.
     */
    public void setvSpeed(int vSpeed) {
        this.vSpeed = vSpeed;
    }

    /**
     * @return the vertical acceleration of the character.
     */
    public int getAccel() {
        return accel;
    }

    /**
     * @param accel  the vertical acceleration of the character.
     */
    public void setAccel(int accel) {
        this.accel = accel;
    }

    /**
     * @return whether or not the charcater is facing left.
     */
    public boolean isLeft() {
        return left;
    }

    /**
     * @param left whether or not the charcater is facing left.
     */
    public void setLeft(boolean left) {
        this.left = left;
    }

    /**
     * @return a String representation of the object.
     */
    public String toString() {
        return "Rabbit{" +
                "speed = " + speed +
                ", frame = " + frame +
                ", steps = " + steps +
                ", vSpeed = " + vSpeed +
                ", accel = " + accel +
                ", idleStart = " + idleStart +
                ", left = " + left +
                '}';
    }
}
